/*! For license information please see 35.720d45ba.chunk.js.LICENSE.txt */
(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[35],{3836:function(t,e,r){"use strict";r.r(e);var i=r(0),n=r.n(i),o=r(3805),a=r(232);var c=Object(a.a)((function(t){var e=t.width,r=t.element,a=t.index,c=t.height,s="bokeh-chart-".concat(a),h=function(t){var i=document.getElementById(s),n=t&&t.doc&&t.doc.roots&&t.doc.roots.references?t.doc.roots.references.find((function(t){return"Plot"===t.type})):void 0;if(n){var a=function(t){var i=t.attributes.plot_width,n=t.attributes.plot_height;return c?(i=e,n=c):r.useContainerWidth&&(i=e),{chartWidth:i,chartHeight:n}}(n),h=a.chartWidth,u=a.chartHeight;h>0&&(n.attributes.plot_width=h),u>0&&(n.attributes.plot_height=u)}null!==i&&(!function(t){for(;t.lastChild;)t.lastChild.remove()}(i),o.a.embed_item(t,s))};return Object(i.useEffect)((function(){h(JSON.parse(r.figure))})),n.a.createElement("div",{id:s,className:"stBokehChart"})}));r.d(e,"default",(function(){return c}))}}]);
//# sourceMappingURL=35.720d45ba.chunk.js.map