(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[23],{1692:function(t,e){},1693:function(t,e){},2271:function(t,e){},3829:function(t,e,i){"use strict";i.r(e);var n=i(0),r=i.n(n),c=i(2251),a=i(2269),h=i(10),o=i(232),s=i(5),u=i.n(s)()("div",{target:"e1p558ko0"})((function(t){return{"& *":{fontFamily:t.theme.fonts.sansSerif,fontSize:"9.6px"},"& svg":{maxWidth:"100%"}}}),"");a.graphviz;var f=Object(o.a)((function(t){var e=t.width,i=t.element,a=t.index,o=t.height,s="graphviz-chart-".concat(a),f=0,d=0,g=function(){var t=d,n=f;return o?(t=e,n=o):i.useContainerWidth&&(t=e),{chartWidth:t,chartHeight:n}},v=function(){try{var t=Object(c.select)("#".concat(s)).graphviz().zoom(!1).fit(!0).scale(1).renderDot(i.spec).on("end",(function(){var t=Object(c.select)("#".concat(s," > svg")).node();t&&(f=t.getBBox().height,d=t.getBBox().width)})),e=g(),n=e.chartHeight,r=e.chartWidth;n>0&&t.height(n),r>0&&t.width(r)}catch(a){Object(h.b)(a)}};Object(n.useEffect)((function(){v()}));var p=g(),l=p.chartWidth?p.chartWidth:e,b=p.chartHeight?p.chartHeight:o;return r.a.createElement(u,{className:"graphviz stGraphVizChart",id:s,style:{width:l,height:b}})}));i.d(e,"default",(function(){return f}))}}]);
//# sourceMappingURL=23.d59db617.chunk.js.map