/*! For license information please see 34.9905a660.chunk.js.LICENSE.txt */
(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[34],{3835:function(t,e,r){"use strict";r.r(e);var s=r(0),n=r.n(s),u=r(57);function c(t){var e=t.element,r=t.width,c=Object(s.useRef)(null);Object(s.useEffect)((function(){c.current&&(c.current.currentTime=e.startTime)}),[e.startTime]);var i=Object(u.b)(e.url);return n.a.createElement("audio",{id:"audio",ref:c,controls:!0,src:i,className:"stAudio",style:{width:r}})}r.d(e,"default",(function(){return c}))}}]);
//# sourceMappingURL=34.9905a660.chunk.js.map