(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[38],{3830:function(t,e,i){"use strict";i.r(e);var n,r=i(0),a=i.n(r),l=i(232),o=i(57),c=i(5),s=i.n(c),d=s()("div",{target:"etr89bj0"})((function(t){return{display:"inline-flex",flexDirection:"column",marginRight:t.theme.fontSizes.smDefault}}),""),h=s()("div",{target:"etr89bj1"})((function(t){var e=t.theme;return{fontFamily:e.fonts.mono,fontSize:e.fontSizes.smDefault,textAlign:"center"}}),"");!function(t){t[t.OriginalWidth=-1]="OriginalWidth",t[t.ColumnWidth=-2]="ColumnWidth"}(n||(n={}));var m=Object(l.a)((function(t){var e,i=t.width,r=t.isFullScreen,l=t.element,c=t.height,s=l.width;if(s===n.OriginalWidth)e=void 0;else if(s===n.ColumnWidth)e=i;else{if(!(s>0))throw Error("Invalid image width: ".concat(s));e=s}var m={};return c&&r?(m.height=c,m["object-fit"]="contain"):m.width=e,a.a.createElement("div",{style:{width:i}},l.imgs.map((function(t,i){var n=t;return a.a.createElement(d,{key:i,"data-testid":"stImage",style:{width:e}},a.a.createElement("img",{style:m,src:Object(o.b)(n.url),alt:i.toString()}),!r&&a.a.createElement(h,{"data-testid":"caption"}," ".concat(n.caption," ")))})))}));i.d(e,"default",(function(){return m}))}}]);
//# sourceMappingURL=38.82f337a7.chunk.js.map