/*! For license information please see 37.71663900.chunk.js.LICENSE.txt */
(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[37],{3837:function(t,e,r){"use strict";r.r(e);var i=r(227),n=r(0),s=r.n(n);function o(t){var e,r,n=t.element,o=t.width,l=n.hasWidth?n.width:o;n.scrolling?(e="auto",r={}):(e="no",r={overflow:"hidden"});var a,h,u=c(n.src);return null!=u?(a=void 0,h=!0):(a=c(n.srcdoc),h=!1),s.a.createElement("iframe",{allow:i.a,style:r,src:u,srcDoc:a,width:l,height:n.height,scrolling:e,sandbox:Object(i.c)(h),title:"st.iframe"})}function c(t){return null==t||""===t?void 0:t}r.d(e,"default",(function(){return o}))}}]);
//# sourceMappingURL=37.71663900.chunk.js.map