/*! For license information please see 43.efded558.chunk.js.LICENSE.txt */
(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[43],{3840:function(e,t,r){"use strict";r.r(t);var i=r(0),a=r.n(i),n=r(22);var s=function(e){var t=e.disabled,r=e.element,i=e.widgetMgr,s={width:e.width};return a.a.createElement("div",{className:"row-widget stButton",style:s},a.a.createElement(n.c,{kind:n.a.PRIMARY,size:n.b.SMALL,disabled:t,onClick:function(){var e=r.id;i.setTriggerValue(e,{fromUi:!0})}},r.label))};r.d(t,"default",(function(){return s}))}}]);
//# sourceMappingURL=43.efded558.chunk.js.map