# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: streamlit/proto/BlockPath.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='streamlit/proto/BlockPath.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1fstreamlit/proto/BlockPath.proto\"r\n\tBlockPath\x12\'\n\tcontainer\x18\x01 \x01(\x0e\x32\x14.BlockPath.Container\x12\x0c\n\x04path\x18\x02 \x03(\r\".\n\tContainer\x12\n\n\x06UNUSED\x10\x00\x12\x08\n\x04MAIN\x10\x01\x12\x0b\n\x07SIDEBAR\x10\x02\x62\x06proto3'
)



_BLOCKPATH_CONTAINER = _descriptor.EnumDescriptor(
  name='Container',
  full_name='BlockPath.Container',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNUSED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MAIN', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SIDEBAR', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=103,
  serialized_end=149,
)
_sym_db.RegisterEnumDescriptor(_BLOCKPATH_CONTAINER)


_BLOCKPATH = _descriptor.Descriptor(
  name='BlockPath',
  full_name='BlockPath',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='container', full_name='BlockPath.container', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='path', full_name='BlockPath.path', index=1,
      number=2, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _BLOCKPATH_CONTAINER,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=35,
  serialized_end=149,
)

_BLOCKPATH.fields_by_name['container'].enum_type = _BLOCKPATH_CONTAINER
_BLOCKPATH_CONTAINER.containing_type = _BLOCKPATH
DESCRIPTOR.message_types_by_name['BlockPath'] = _BLOCKPATH
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

BlockPath = _reflection.GeneratedProtocolMessageType('BlockPath', (_message.Message,), {
  'DESCRIPTOR' : _BLOCKPATH,
  '__module__' : 'streamlit.proto.BlockPath_pb2'
  # @@protoc_insertion_point(class_scope:BlockPath)
  })
_sym_db.RegisterMessage(BlockPath)


# @@protoc_insertion_point(module_scope)
