# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: streamlit/proto/ArrowDataframe.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from streamlit.proto import ArrowTable_pb2 as streamlit_dot_proto_dot_ArrowTable__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='streamlit/proto/ArrowDataframe.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n$streamlit/proto/ArrowDataframe.proto\x1a streamlit/proto/ArrowTable.proto\"J\n\x0e\x41rrowDataframe\x12\x19\n\x04\x64\x61ta\x18\x01 \x01(\x0b\x32\x0b.ArrowTable\x12\x0e\n\x06height\x18\x02 \x01(\r\x12\r\n\x05width\x18\x03 \x01(\rb\x06proto3'
  ,
  dependencies=[streamlit_dot_proto_dot_ArrowTable__pb2.DESCRIPTOR,])




_ARROWDATAFRAME = _descriptor.Descriptor(
  name='ArrowDataframe',
  full_name='ArrowDataframe',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='ArrowDataframe.data', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='height', full_name='ArrowDataframe.height', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='width', full_name='ArrowDataframe.width', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=74,
  serialized_end=148,
)

_ARROWDATAFRAME.fields_by_name['data'].message_type = streamlit_dot_proto_dot_ArrowTable__pb2._ARROWTABLE
DESCRIPTOR.message_types_by_name['ArrowDataframe'] = _ARROWDATAFRAME
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ArrowDataframe = _reflection.GeneratedProtocolMessageType('ArrowDataframe', (_message.Message,), {
  'DESCRIPTOR' : _ARROWDATAFRAME,
  '__module__' : 'streamlit.proto.ArrowDataframe_pb2'
  # @@protoc_insertion_point(class_scope:ArrowDataframe)
  })
_sym_db.RegisterMessage(ArrowDataframe)


# @@protoc_insertion_point(module_scope)
