(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[39],{3838:function(t,e,r){"use strict";r.r(e);var a=r(0),n=r.n(a),i=r(232),u=r(2272),c=r.n(u);var o=Object(i.a)((function(t){var e=t.width,r=t.element,a=t.height,i=function(){return!!a};switch(r.chart){case"url":return function(t){var r=a||450,i=e;return n.a.createElement("iframe",{title:"Plotly",src:t,style:{width:i,height:r}})}(r.url);case"figure":return function(t){var u=JSON.parse(t.config),o=function(t){var n=JSON.parse(t.spec);return i()?(n.layout.width=e,n.layout.height=a):r.useContainerWidth&&(n.layout.width=e),n}(t),l=o.data,s=o.layout,h=o.frames;return n.a.createElement(c.a,{key:i()?"fullscreen":"original",className:"stPlotlyChart",data:l,layout:s,config:u,frames:h})}(r.figure);default:throw new Error("Unrecognized PlotlyChart type: ".concat(r.chart))}}));r.d(e,"default",(function(){return o}))}}]);
//# sourceMappingURL=39.c7694cc9.chunk.js.map