/*! For license information please see 42.85ad720c.chunk.js.LICENSE.txt */
(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[42],{3839:function(e,t,r){"use strict";r.r(t);var a=r(0),n=r.n(a),c=r(11),i=r(57);function s(e){var t=e.element,r=e.width,s=Object(a.useRef)(null),o=t.type,u=t.url;Object(a.useEffect)((function(){var e=s.current,r=function(){e&&(e.currentTime=t.startTime)};return e&&e.addEventListener("loadedmetadata",r),function(){e&&e.removeEventListener("loadedmetadata",r)}}),[t]);if(o===c.q.Type.YOUTUBE_IFRAME){var l=.75*r;return n.a.createElement("iframe",{title:u,src:function(e){var r=t.startTime;return r?"".concat(e,"?start=").concat(r):e}(u),width:r,height:l,frameBorder:"0",allow:"autoplay; encrypted-media",allowFullScreen:!0})}return n.a.createElement("video",{ref:s,controls:!0,src:Object(i.b)(u),className:"stVideo",style:{width:r}})}r.d(t,"default",(function(){return s}))}}]);
//# sourceMappingURL=42.85ad720c.chunk.js.map