# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: streamlit/proto/SessionEvent.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from streamlit.proto import Exception_pb2 as streamlit_dot_proto_dot_Exception__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='streamlit/proto/SessionEvent.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\"streamlit/proto/SessionEvent.proto\x1a\x1fstreamlit/proto/Exception.proto\"\x93\x01\n\x0cSessionEvent\x12 \n\x16report_changed_on_disk\x18\x01 \x01(\x08H\x00\x12%\n\x1breport_was_manually_stopped\x18\x02 \x01(\x08H\x00\x12\x32\n\x1cscript_compilation_exception\x18\x03 \x01(\x0b\x32\n.ExceptionH\x00\x42\x06\n\x04typeb\x06proto3'
  ,
  dependencies=[streamlit_dot_proto_dot_Exception__pb2.DESCRIPTOR,])




_SESSIONEVENT = _descriptor.Descriptor(
  name='SessionEvent',
  full_name='SessionEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='report_changed_on_disk', full_name='SessionEvent.report_changed_on_disk', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='report_was_manually_stopped', full_name='SessionEvent.report_was_manually_stopped', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='script_compilation_exception', full_name='SessionEvent.script_compilation_exception', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='type', full_name='SessionEvent.type',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=72,
  serialized_end=219,
)

_SESSIONEVENT.fields_by_name['script_compilation_exception'].message_type = streamlit_dot_proto_dot_Exception__pb2._EXCEPTION
_SESSIONEVENT.oneofs_by_name['type'].fields.append(
  _SESSIONEVENT.fields_by_name['report_changed_on_disk'])
_SESSIONEVENT.fields_by_name['report_changed_on_disk'].containing_oneof = _SESSIONEVENT.oneofs_by_name['type']
_SESSIONEVENT.oneofs_by_name['type'].fields.append(
  _SESSIONEVENT.fields_by_name['report_was_manually_stopped'])
_SESSIONEVENT.fields_by_name['report_was_manually_stopped'].containing_oneof = _SESSIONEVENT.oneofs_by_name['type']
_SESSIONEVENT.oneofs_by_name['type'].fields.append(
  _SESSIONEVENT.fields_by_name['script_compilation_exception'])
_SESSIONEVENT.fields_by_name['script_compilation_exception'].containing_oneof = _SESSIONEVENT.oneofs_by_name['type']
DESCRIPTOR.message_types_by_name['SessionEvent'] = _SESSIONEVENT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SessionEvent = _reflection.GeneratedProtocolMessageType('SessionEvent', (_message.Message,), {
  'DESCRIPTOR' : _SESSIONEVENT,
  '__module__' : 'streamlit.proto.SessionEvent_pb2'
  # @@protoc_insertion_point(class_scope:SessionEvent)
  })
_sym_db.RegisterMessage(SessionEvent)


# @@protoc_insertion_point(module_scope)
